@@ -1,6 +1,8 @@
 use self::ctx::Ctx;
+use crate::marks::Marks;
 use crate::util::can_end_conditionally;
 use crate::util::idents_used_by;
+use crate::util::now;
 use fxhash::FxHashMap;
 use fxhash::FxHashSet;
 use std::collections::hash_map::Entry;
@@ -21,14 +23,15 @@ mod ctx;
 
 /// TODO: Track assignments to variables via `arguments`.
 /// TODO: Scope-local. (Including block)
-pub(crate) fn analyze<N>(n: &N) -> ProgramData
+pub(crate) fn analyze<N>(n: &N, marks: Marks) -> ProgramData
 where
     N: VisitWith<UsageAnalyzer>,
 {
-    let start_time = Instant::now();
+    let start_time = now();
 
     let mut v = UsageAnalyzer {
         data: Default::default(),
+        marks,
         scope: Default::default(),
         ctx: Default::default(),
     };
@@ -36,9 +39,11 @@ where
     let top_scope = v.scope;
     v.data.top.merge(top_scope, false);
 
-    let end_time = Instant::now();
+    if let Some(start_time) = start_time {
+        let end_time = Instant::now();
 
-    log::debug!("Scope analysis took {:?}", end_time - start_time);
+        log::debug!("Scope analysis took {:?}", end_time - start_time);
+    }
 
     v.data
 }
@@ -232,6 +237,7 @@ impl ProgramData {
 #[derive(Debug)]
 pub(crate) struct UsageAnalyzer {
     data: ProgramData,
+    marks: Marks,
     scope: ScopeData,
     ctx: Ctx,
 }
@@ -243,6 +249,7 @@ impl UsageAnalyzer {
     {
         let mut child = UsageAnalyzer {
             data: Default::default(),
+            marks: self.marks,
             ctx: self.ctx,
             scope: Default::default(),
         };
@@ -272,6 +279,7 @@ impl UsageAnalyzer {
             // log::trace!("insert({}{:?})", i.0, i.1);
 
             VarUsageInfo {
+                is_fn_local: true,
                 used_above_decl: true,
                 ..Default::default()
             }
@@ -320,11 +328,15 @@ impl UsageAnalyzer {
             .vars
             .entry(i.to_id())
             .and_modify(|v| {
-                if has_init {
+                if has_init && v.declared {
                     v.mutated = true;
                     v.reassigned = true;
                     v.assign_count += 1;
                 }
+
+                if v.used_by_nested_fn {
+                    v.is_fn_local = false;
+                }
             })
             .or_insert_with(|| VarUsageInfo {
                 is_fn_local: true,
@@ -388,7 +400,12 @@ impl Visit for UsageAnalyzer {
         };
         n.left.visit_with(n, &mut *self.with_ctx(ctx));
 
-        n.right.visit_with(n, self);
+        let ctx = Ctx {
+            in_assign_lhs: false,
+            is_exact_reassignment: false,
+            ..self.ctx
+        };
+        n.right.visit_with(n, &mut *self.with_ctx(ctx));
     }
 
     fn visit_block_stmt(&mut self, n: &BlockStmt, _: &dyn Node) {
@@ -398,11 +415,22 @@ impl Visit for UsageAnalyzer {
     }
 
     fn visit_call_expr(&mut self, n: &CallExpr, _: &dyn Node) {
+        let inline_prevented = self.ctx.inline_prevented || n.span.has_mark(self.marks.noinline);
+
         {
-            n.callee.visit_with(n, self);
             let ctx = Ctx {
+                inline_prevented,
+                ..self.ctx
+            };
+            n.callee.visit_with(n, &mut *self.with_ctx(ctx));
+        }
+
+        {
+            let ctx = Ctx {
+                inline_prevented,
                 in_call_arg: true,
                 is_exact_arg: true,
+                is_exact_reassignment: false,
                 ..self.ctx
             };
             n.args.visit_with(n, &mut *self.with_ctx(ctx));
@@ -750,15 +778,15 @@ impl Visit for UsageAnalyzer {
     fn visit_stmts(&mut self, stmts: &[Stmt], _: &dyn Node) {
         let mut had_cond = false;
 
-        for n in stmts {
+        for stmt in stmts {
             let ctx = Ctx {
                 in_cond: self.ctx.in_cond || had_cond,
                 ..self.ctx
             };
 
-            n.visit_with(&Invalid { span: DUMMY_SP }, &mut *self.with_ctx(ctx));
+            stmt.visit_with(&Invalid { span: DUMMY_SP }, &mut *self.with_ctx(ctx));
 
-            had_cond |= can_end_conditionally(n);
+            had_cond |= can_end_conditionally(stmt);
         }
     }
 
