@@ -1,9 +1,10 @@
-use std::time::Instant;
-
 use crate::analyzer::analyze;
 use crate::analyzer::ProgramData;
+use crate::marks::Marks;
 use crate::pass::hygiene::analyzer::HygieneAnalyzer;
 use crate::pass::hygiene::analyzer::HygieneData;
+use crate::util::now;
+use std::time::Instant;
 use swc_common::Mark;
 use swc_common::SyntaxContext;
 use swc_common::DUMMY_SP;
@@ -16,8 +17,11 @@ use swc_ecma_visit::VisitWith;
 
 mod analyzer;
 
+/// Optimize hygiene info to get minified output.
+///
+/// Requires [swc_common::GLOBALS].
 pub fn optimize_hygiene(m: &mut Module, top_level_mark: Mark) {
-    let data = analyze(&*m);
+    let data = analyze(&*m, Marks::new());
     m.visit_mut_with(&mut hygiene_optimizer(data, top_level_mark))
 }
 
@@ -70,7 +74,7 @@ impl VisitMut for Optimizer {
 
     fn visit_mut_module(&mut self, n: &mut Module) {
         log::info!("hygiene: Analyzing span hygiene");
-        let start = Instant::now();
+        let start = now();
 
         let mut analyzer = HygieneAnalyzer {
             data: &self.data,
@@ -81,13 +85,18 @@ impl VisitMut for Optimizer {
         n.visit_with(&Invalid { span: DUMMY_SP }, &mut analyzer);
         self.hygiene = analyzer.hygiene;
 
-        let end = Instant::now();
-        log::info!("hygiene: Span hygiene analysis took {:?}", end - start);
-        let start = end;
+        if let Some(start) = start {
+            let end = Instant::now();
+            log::info!("hygiene: Span hygiene analysis took {:?}", end - start);
+        }
+        let start = now();
 
         log::info!("hygiene: Optimizing span hygiene");
         n.visit_mut_children_with(self);
-        let end = Instant::now();
-        log::info!("hygiene: Span hygiene optimiation took {:?}", end - start);
+
+        if let Some(start) = start {
+            let end = Instant::now();
+            log::info!("hygiene: Span hygiene optimiation took {:?}", end - start);
+        }
     }
 }
