@@ -2,8 +2,9 @@ use self::preserver::idents_to_preserve;
 use super::compute_char_freq::CharFreqInfo;
 use crate::analyzer::analyze;
 use crate::analyzer::ProgramData;
+use crate::marks::Marks;
 use crate::option::MangleOptions;
-use crate::util::base54::base54;
+use crate::util::base54::incr_base54;
 use fxhash::FxHashMap;
 use fxhash::FxHashSet;
 use swc_atoms::JsWord;
@@ -17,14 +18,25 @@ use swc_ecma_visit::VisitMutWith;
 
 mod preserver;
 
-pub fn name_mangler(options: MangleOptions, _char_freq_info: CharFreqInfo) -> impl VisitMut {
+pub(crate) fn name_mangler(
+    options: MangleOptions,
+    _char_freq_info: CharFreqInfo,
+    marks: Marks,
+) -> impl VisitMut {
     Mangler {
         options,
-        ..Default::default()
+        n: 0,
+        private_n: 0,
+        preserved: Default::default(),
+        preserved_symbols: Default::default(),
+        renamed: Default::default(),
+        renamed_private: Default::default(),
+        data: Default::default(),
+        marks,
     }
 }
 
-#[derive(Debug, Default)]
+#[derive(Debug)]
 struct Mangler {
     options: MangleOptions,
     n: usize,
@@ -35,6 +47,8 @@ struct Mangler {
     renamed: FxHashMap<Id, JsWord>,
     renamed_private: FxHashMap<Id, JsWord>,
     data: Option<ProgramData>,
+
+    marks: Marks,
 }
 
 impl Mangler {
@@ -56,8 +70,9 @@ impl Mangler {
         }
 
         loop {
-            let sym: JsWord = base54(self.n).into();
-            self.n += 1;
+            let sym = incr_base54(&mut self.n);
+
+            let sym: JsWord = sym.into();
             if self.preserved_symbols.contains(&sym) {
                 continue;
             }
@@ -75,12 +90,15 @@ impl Mangler {
         let new_sym = if let Some(cached) = self.renamed_private.get(&id) {
             cached.clone()
         } else {
-            let sym: JsWord = base54(self.private_n).into();
-            self.private_n += 1;
+            loop {
+                let sym = incr_base54(&mut self.private_n);
 
-            self.renamed_private.insert(id.clone(), sym.clone());
+                let sym: JsWord = sym.into();
 
-            sym
+                self.renamed_private.insert(id.clone(), sym.clone());
+
+                break sym;
+            }
         };
 
         private_name.id.sym = new_sym;
@@ -149,7 +167,7 @@ impl VisitMut for Mangler {
     }
 
     fn visit_mut_module(&mut self, n: &mut Module) {
-        let data = analyze(&*n);
+        let data = analyze(&*n, self.marks);
         self.data = Some(data);
         self.preserved = idents_to_preserve(self.options.clone(), n);
         self.preserved_symbols = self.preserved.iter().map(|v| v.0.clone()).collect();
@@ -182,7 +200,7 @@ impl VisitMut for Mangler {
     }
 
     fn visit_mut_script(&mut self, n: &mut Script) {
-        let data = analyze(&*n);
+        let data = analyze(&*n, self.marks);
         self.data = Some(data);
         self.preserved = idents_to_preserve(self.options.clone(), n);
         self.preserved_symbols = self.preserved.iter().map(|v| v.0.clone()).collect();
